--------------------------------------------------------------------------
-- Top-level instantiation of the MR4 Datapath and Control Unit
--------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_Logic_unsigned.all;
use work.p_MR4.all;

entity MR4_with_memories is
	port
	(
		clock, reset, selCPU: in std_logic;
		addressSERIAL: in reg32;
		data_out: out reg32
	);
end MR4_with_memories;

architecture a1 of MR4_with_memories is

	signal IR: reg32;
	signal uins: microinstruction;
	signal end_mul, end_div, rst_cpu: std_logic;
	signal int_address : std_logic_vector(8 downto 0);    -- 9 bits endereveis : 512 words
	signal i_address, instruction, addressCPU, addressRAM, data_out_CPU, data_out_RAM : reg32;

begin

	dp: entity work.datapath port map
	(
		ck=>clock,
		rst=>rst_cpu,
		end_mul=>end_mul,
		end_div =>end_div,
		IR_OUT=>IR,
		uins=>uins,
		i_address=>i_address,
		instruction=>instruction,
		d_address=>addressCPU,
		data_out=>data_out_CPU,
		data_in=>data_out_RAM
	);

	ct: entity work.control_unit port map
	(
		ck=>clock,
		rst=>rst_cpu,
		end_mul=>end_mul,
		end_div=>end_div,
		IR=>IR,
		uins=>uins
	);

	-----
	----  MEMRIA RAM COM AS INSTRUES
	----
	int_address <=  i_address(10 downto 2) - 8;   -- tira o deslocamento de 20H
	m1: entity work.program_memory port map
	(
		clock=>clock,
		address=>int_address,
		instruction=>instruction
	);

	----
	----  MEMRIA RAM COM OS DADOS
	----
	m2: entity work.data_memory port map
	(
		clock=>clock,
		ce=>uins.ce,
		we=> uins.rw,
		bw=>uins.bw,
		address=>addressRAM(12 downto 2),
		data_in=>data_out_CPU,
		data_out=>data_out_RAM
	);

	addressRAM <= addressCPU when selCPU='0' else addressSERIAL;
	rst_cpu <= reset or selCPU;
	data_out <= data_out_RAM;

end a1;